@@ -4,8 +4,6 @@ pub mod utils;
 
 use serde_json::json;
 use serde_json::to_string;
-use std::fs::create_dir_all;
-use std::fs::File;
 use std::fs::OpenOptions;
 use std::io::Write;
 pub use types::*;
