@@ -1,4 +1,3 @@
-#![allow(non_snake_case)]
 use reqwest::blocking::Client;
 use serde::Deserialize;
 use serde::Serialize;
@@ -9,12 +8,13 @@ use std::path::PathBuf;
 use std::process::Child as _Child;
 use url::Url;
 
+#[allow(non_snake_case)]
 #[derive(Clone, Serialize, Deserialize, PartialEq, Debug)]
 pub struct PlayItOpts {
   pub PREFERRED_TUNNEL: Option<String>,
-  pub PREFERRED_THRESHOLD: Option<i64>,
+  pub PREFERRED_THRESHOLD: Option<u16>,
   pub NO_SPECIAL_LAN: Option<bool>,
-  pub KEEP_CONFIG: Option<bool>,
+  // pub KEEP_CONFIG: Option<bool>,
 }
 
 #[derive(Clone, Serialize, Deserialize, PartialEq, Debug)]
@@ -40,14 +40,15 @@ pub struct Tunnels {
 pub struct Connection {
   pub ip: String,
   pub tunnel: Tunnel,
-  pub proto: String,
+  pub proto: Prototype,
 }
 
+#[derive(Debug)]
 pub struct PlayIt {
   pub req_client: Client,
-  pub os: String,
+  pub os: OS,
   pub config_file: PathBuf,
-  pub arch: String,
+  pub arch: Arch,
   pub dir: PathBuf,
   pub destroyed: bool,
   pub tunnels: Vec<Tunnel>,
@@ -61,7 +62,7 @@ pub struct PlayIt {
   pub version: String,
   pub download_urls: Binaries,
   pub binary: PathBuf,
-  pub binary_type: String,
+  pub binary_type: BinaryType,
   pub output: Vec<String>,
   pub stdout: Vec<String>,
   pub stderr: Vec<String>,
@@ -70,13 +71,6 @@ pub struct PlayIt {
   pub api_path: Url,
 }
 
-#[derive(Clone, Serialize, Deserialize, PartialEq, Debug)]
-pub struct CreateTunnelOptions {
-  pub proto: String,
-  pub ip: Option<String>,
-  pub port: u16,
-}
-
 #[derive(Clone, Serialize, Deserialize, PartialEq, Debug)]
 pub struct Agent {
   pub tunnel_server_name: Option<String>,
@@ -100,17 +94,17 @@ pub struct Binaries {
   pub arm: Url,
 }
 
-impl Index<&'_ str> for Binaries {
+impl Index<BinaryType> for Binaries {
   type Output = Url;
 
-  fn index(&self, binary: &'_ str) -> &Self::Output {
-    match binary {
-      "win" => &self.win,
-      "lin" => &self.lin,
-      "mac" => &self.mac,
-      "arm64" => &self.arm64,
-      "arm" => &self.arm,
-      _ => panic!("Unknown Index: {}", binary),
+  fn index(&self, binary_type: BinaryType) -> &Self::Output {
+    match binary_type {
+      BinaryType::Windows => &self.win,
+      BinaryType::Linux => &self.lin,
+      BinaryType::MacOS => &self.mac,
+      BinaryType::Arm64 => &self.arm64,
+      BinaryType::Arm => &self.arm,
+      _ => panic!("Unknown Index: {:#?}", binary_type),
     }
   }
 }
@@ -126,6 +120,64 @@ pub struct Id {
   pub id: u32,
 }
 
+#[derive(Clone, Serialize, Deserialize, PartialEq, Debug)]
+pub enum Prototype {
+  Tcp,
+  Udp,
+}
+
+#[derive(Clone, Serialize, Deserialize, PartialEq, Debug)]
+pub enum OS {
+  Windows,
+  Linux,
+  MacOS,
+}
+
+#[derive(Clone, Serialize, Deserialize, PartialEq, Debug)]
+pub enum Arch {
+  X64,
+  Arm64,
+  Arm,
+  Unsupported,
+}
+
+#[derive(Clone, Serialize, Deserialize, PartialEq, Debug)]
+pub enum BinaryType {
+  Windows,
+  Linux,
+  MacOS,
+  Arm64,
+  Arm,
+  Unsupported,
+}
+
+#[allow(non_camel_case_types)]
+pub enum Servers {
+  dal4,
+  sol4,
+  syd4,
+  mum4,
+  sf4,
+  fnk4,
+  bng4,
+  sng4,
+  tor4,
+  ny4,
+  uk4,
+  saw4,
+  turk4,
+  san4,
+  pet4,
+  bur4,
+  new4,
+  isr4,
+  tko4,
+  syd5,
+  sng5,
+  hel4,
+  fal4,
+}
+
 // https://doc.rust-lang.org/std/process/struct.Child.html
 /// Representation of a running or exited child process.
 ///
@@ -175,7 +227,7 @@ pub struct Id {
 pub struct Child(pub _Child);
 
 impl Drop for Child {
-  fn drop(&mut self) -> () {
+  fn drop(&mut self) {
     self.0.kill().ok();
   }
 }
@@ -184,12 +236,12 @@ impl Deref for Child {
   type Target = _Child;
 
   fn deref(&self) -> &Self::Target {
-    return &self.0;
+    &self.0
   }
 }
 
 impl DerefMut for Child {
   fn deref_mut(&mut self) -> &mut _Child {
-    return &mut self.0;
+    &mut self.0
   }
 }
